using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Blazored.Modal;
using Blazorise;
using Blazorise.Bulma;
using Blazorise.Icons.FontAwesome;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Components;
using Microsoft.AspNetCore.Hosting;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using Seenginx.Services;
using Seenginx.Services.Models;

namespace Seenginx
{
	public class Startup
	{
		public Startup(IConfiguration configuration)
		{
			Configuration = configuration;
		}

		public IConfiguration Configuration { get; }

		public void ConfigureServices(IServiceCollection services)
		{
			services.AddRazorPages()
#if DEBUG
				.AddRazorRuntimeCompilation()
#endif
				;
			services.AddServerSideBlazor();
			services.AddBlazorise(options =>
			{
				options.ChangeTextOnKeyPress = true; // optional
			}).AddBulmaProviders();
			services.AddBlazoredModal();

			var configPaths = new ConfigPaths();
			configPaths.NginxPath = @"C:\nginx\";
			configPaths.SystemDPath = @"C:\systemd\system\";

			services.AddSingleton(configPaths);
			services.AddTransient<IDmesgService, DmesgService>();
			services.AddTransient<INginxService, NginxService>();
			services.AddTransient<ISystemDService, SystemDService>();
			services.AddTransient<IFileManager, FileManager>();
		}

		// This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
		public void Configure(IApplicationBuilder app, IWebHostEnvironment env)
		{
			if (env.IsDevelopment())
			{
				app.UseDeveloperExceptionPage();
			}
			else
			{
				app.UseExceptionHandler("/Error");
			}

			app.UseStaticFiles();

			app.UseRouting();

			app.ApplicationServices.UseBulmaProviders();

			app.UseAuthentication();

			app.UseEndpoints(endpoints =>
			{
				endpoints.MapBlazorHub();
				endpoints.MapFallbackToPage("/_Host");
			});
		}
	}
}
